package io.zipcoder.tc_spring_poll_application.controller;

import io.zipcoder.tc_spring_poll_application.domain.Poll;
import io.zipcoder.tc_spring_poll_application.domain.repositories.PollRepository;
import io.zipcoder.tc_spring_poll_application.exception.ResourceNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import javax.validation.Valid;
import java.net.URI;

/*
Create a `PollController` class in the `controller` sub package.
	* `PollController` signature should be `annotated` with `@RestController`
	* PollController` has a `pollRepository` instance variable of type `PollRepository`
	* Create a constructor that accepts a `PollRepository` argument and assigns its
	* value to the `pollRepository` member variable.
	* Mark the constructor with the `@Autowired` annotation.

 */
@RestController
public class PollController {


    private PollRepository pollRepository;


    //    @Autowired
//    public PollController(PollRepository pollRepository1) {
//        this.pollRepository = pollRepository1;
//
//    }
//
//    @RequestMapping(value = "/polls", method = RequestMethod.GET)
//    public ResponseEntity<Iterable<Poll>> getAllPolls() {
//        Iterable<Poll> allPolls = pollRepository.findAll();
//        return new ResponseEntity<>(allPolls, HttpStatus.OK);
//    }
//
//    @RequestMapping(value = "/polls", method = RequestMethod.POST)
//
//    public ResponseEntity<?> createPoll(@RequestBody Poll poll) {
//        poll = pollRepository.save(poll);
//
//        URI newPollUri = ServletUriComponentsBuilder
//                .fromCurrentRequest()
//                .path("/{id}")
//                .buildAndExpand(poll.getId())
//                .toUri();
//
//        return new ResponseEntity<>(null, HttpStatus.CREATED);
//    }
//
//    @RequestMapping(value = "/polls/{pollId}", method = RequestMethod.GET)
//    public ResponseEntity<?> getPoll(@PathVariable Long pollId) {
//        Poll p = pollRepository.findOne(pollId);
//        return new ResponseEntity<>(p, HttpStatus.OK);
//    }
//
//    @RequestMapping(value = "/polls/{pollId}", method = RequestMethod.PUT)
//    public ResponseEntity<?> updatePoll(@RequestBody Poll poll, @PathVariable Long pollId) {
//        // Save the entity
//        Poll p = pollRepository.save(poll);
//        return new ResponseEntity<>(HttpStatus.OK);
//    }
//
//    @RequestMapping(value = "/polls/{pollId}", method = RequestMethod.DELETE)
//    public ResponseEntity<?> deletePoll(@PathVariable Long pollId) {
//        pollRepository.delete(pollId);
//        return new ResponseEntity<>(HttpStatus.OK);
//    }
//
////    public void verifypoll() throws ResourceNotFoundException {
////        throw new ResourceNotFoundException();
////    }
//}
    public void pollExists(Long pollId) throws ResourceNotFoundException {
        if (pollRepository.findOne(pollId) == null) {
            throw new ResourceNotFoundException("The poll specified does not exist.");
        }

    }

    @Autowired
    public PollController(PollRepository pollRepository) {
        this.pollRepository = pollRepository;
    }

    @GetMapping("/polls")
    public ResponseEntity<Iterable<Poll>> getPolls() {
        Iterable<Poll> allPolls = pollRepository.findAll();
        return new ResponseEntity<>(allPolls, HttpStatus.OK);
    }

    @GetMapping("/polls/{size}/{pageNum}")
    public ResponseEntity<Page<Poll>> getPollsPaged(@PathVariable int size, @PathVariable int pageNum) {

        PageRequest pageRequest = new PageRequest(pageNum, size);
        Page<Poll> page = pollRepository.findAll(pageRequest);

        return new ResponseEntity<>(page, HttpStatus.OK);
    }
        @PostMapping("/polls")
        @Valid
        public ResponseEntity<?> createPoll (@RequestBody Poll poll){
            poll = pollRepository.save(poll);
            HttpHeaders httpHeaders = new HttpHeaders();
            httpHeaders.setLocation(ServletUriComponentsBuilder
                    .fromCurrentRequest()
                    .path("/{id}")
                    .buildAndExpand(poll.getId())
                    .toUri());
            return new ResponseEntity<>(null, HttpStatus.CREATED);
        }

        @GetMapping("/polls/{pollId}")
        public ResponseEntity<?> getPoll (@PathVariable Long pollId){
            Poll p = pollRepository.findOne(pollId);
            return new ResponseEntity<>(p, HttpStatus.OK);

        }

        @PutMapping("/polls/{pollId}")
        @Valid
        public ResponseEntity<?> updatePoll (@RequestBody Poll poll, @PathVariable Long pollId){
            Poll p = pollRepository.save(poll);
            return new ResponseEntity<>(HttpStatus.OK);
        }

        @DeleteMapping("/polls/{pollId}")
        public ResponseEntity<?> deletePoll (@PathVariable Long pollId){
            pollRepository.delete(pollId);
            return new ResponseEntity<>(HttpStatus.OK);
        }
    }


